import waba.ui.*;
import waba.sys.Time;
import extra.io.*;
import extra.io.builtin.*;

/**
 * Tests the access classes for the Palm builtin databases - Datebook,
 * Address, ToDo, and Memo.  Click on the labels of each db to go to the
 * next one.
 */
public class BuiltinTest extends MainWindow
{
  final static String[] months=
    {"January","February","March","April","May","June","July","August",
     "September","October","November","December"};
  final static String[] longdays=
    {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
  final static String[] shortdays=
    {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

  Datebook date=new Datebook();
  Label datebookLabel;
  Edit datebookEdit;
  Edit datebookEdit2;
  int datebookIndex=-1;

  Address address=new Address();
  Label addressLabel;
  Edit addressEdit;
  int addressIndex=-1;

  ToDo todo=new ToDo();
  Label todoLabel;
  Edit todoEdit;
  int todoIndex=-1;

  Memo memo=new Memo();
  Label memoLabel;
  Edit memoEdit;
  int memoIndex=-1;

  public BuiltinTest()
  {
    datebookLabel=new Label("Datebook");
    datebookLabel.setRect(0,0,160,20);
    add(datebookLabel);
    datebookEdit=new Edit();
    datebookEdit.setRect(0,20,160,10);
    add(datebookEdit);
    datebookEdit2=new Edit();
    datebookEdit2.setRect(0,30,160,10);
    add(datebookEdit2);
    incrementDatebook();
    
    addressLabel=new Label("Address");
    addressLabel.setRect(0,40,160,20);
    add(addressLabel);
    addressEdit=new Edit();
    addressEdit.setRect(0,60,160,20);
    add(addressEdit);
    incrementAddress();

    todoLabel=new Label("ToDo");
    todoLabel.setRect(0,80,160,20);
    add(todoLabel);
    todoEdit=new Edit();
    todoEdit.setRect(0,100,160,20);
    add(todoEdit);
    incrementToDo();

    memoLabel=new Label("Memo");
    memoLabel.setRect(0,120,160,20);
    add(memoLabel);
    memoEdit=new Edit();
    memoEdit.setRect(0,140,160,20);
    add(memoEdit);
    incrementMemo();

    /*
    // example of adding entries
    date.description="Sample";
    date.startDate=new Time();
    date.endTime=new Time();
    date.startDate.minute=0;
    date.endTime.minute=30;
    date.alarmUnits=Datebook.ALARM_HOURS;
    date.alarmAdvance=2;
    date.repeatType=Datebook.REPEAT_WEEKLY;
    date.repeatFrequency=2;
    date.repeatWeekdays=new boolean[]{true,true,true,false,true,false,false};
    Datebook.addDate(date);    
    */
  }

  public void onEvent(Event e)
  {
    if (e.type==PenEvent.PEN_DOWN)
    {
      if (e.target==datebookLabel)
        incrementDatebook();
      else
      if (e.target==addressLabel)
        incrementAddress();
      else
      if (e.target==todoLabel)
        incrementToDo();
      else
      if (e.target==memoLabel)
        incrementMemo();
    }
  }

  /**
   * Goes to the next valid date
   */
  public void incrementDatebook()
  {
    int datebookCount=Datebook.datebookCount();
    if (datebookIndex>=datebookCount)
      return;

    int lastIndex=datebookIndex;    
    do
    {
      datebookIndex++;
      if (datebookIndex==datebookCount)
        if (lastIndex==-1)
        {
          datebookEdit.setText("No dates");
          return;
        }
        else
          datebookIndex=0;
    }
    while(!Datebook.getDate(datebookIndex,date));

    datebookLabel.setText("Datebook "+(datebookIndex+1)+" / "+datebookCount);
    StringBuffer sb=new StringBuffer();
    sb.append(date.description).append(" @ ");
    Time start=date.startDate;
    if (date.endTime!=null) // there is a time
      sb.append(start.hour).append(':').append(start.minute).
         append((start.minute<10)?"0 ":" ");
    sb.append(start.day).append('/').append(start.month).append('/').
      append(start.year).append(' ');
    if (date.note!=null)
      sb.append("(Note)");

    datebookEdit.setText(sb.toString());
    sb=new StringBuffer();
    if (date.alarmAdvance!=-1)
    {
      sb.append("Alarm ").append(date.alarmAdvance);
      switch(date.alarmUnits)
      {
        case Datebook.ALARM_MINUTES: sb.append(" mins "); break;
        case Datebook.ALARM_HOURS: sb.append(" hrs "); break;
        case Datebook.ALARM_DAYS: sb.append(" days "); break;
      }
    }
    sb.append("Repeat ");
    switch(date.repeatType)
    {
      case Datebook.REPEAT_NONE:
        sb.append("none");
        break;
      case Datebook.REPEAT_DAILY:
        sb.append("every ");
        appendDigit(sb,date.repeatFrequency,true);
        sb.append("day");
        break;
      case Datebook.REPEAT_WEEKLY:
        sb.append("every ");
        appendDigit(sb,date.repeatFrequency,true);
        sb.append("week on ");
        int cnt=0;
        for(int i=0;i<7;i++)
          if (date.repeatWeekdays[i])
            cnt++;
        int pos=0;
        for(int i=0;i<7;i++)
          if (date.repeatWeekdays[i])
          {
            pos++;
            if (cnt==1)
            {
              sb.append(longdays[i]);
              return;
            }
            if (pos==cnt)
              sb.append(" and ");
            sb.append(shortdays[i]);
            if (pos<cnt&&cnt>2)
              sb.append(", ");
          }
        break;
      case Datebook.REPEAT_MONTHLY_BY_DAY:
        sb.append("on the ");
        appendDigit(sb,date.repeatMonthlyCount,false);
        sb.append(longdays[date.repeatMonthlyDay]).append(" of every ");
        appendDigit(sb,date.repeatFrequency,true);
        sb.append("month");
        break;
      case Datebook.REPEAT_MONTHLY_BY_DATE:
        sb.append("on the ");
        appendDigit(sb,date.startDate.day,false);
        sb.append("of every ");
        appendDigit(sb,date.repeatFrequency,true);
        sb.append("month");
        break;
      case Datebook.REPEAT_YEARLY:
        sb.append(months[date.startDate.month-1]).append(" every ");
        appendDigit(sb,date.repeatFrequency,true);
        sb.append("year");
        break;
    }
    datebookEdit2.setText(sb.toString());
  }

  /**
   * Appends a number to the string buffer as an index. ie number 23 gets
   * added as '23rd ' and 5 gets added as '5th '.
   * @param sb the buffer to add to
   * @param number the number to add
   * @param fixOneAndTwo, if true, don't write anything for 1 and write
   * 'other ' for two.  This way you can write 'every other day'.  If
   * false, the standard rules apply.
   */
  public void appendDigit(StringBuffer sb,int number,boolean fixOneAndTwo)
  {
    if (fixOneAndTwo)
    {
      if (number==1)
        return;
      if (number==2)
      {
        sb.append("other ");
        return;
      }
    }
    sb.append(number);
    int tens=number/10;
    int lastDigit=number%10;
    if (lastDigit==1)
    {
      sb.append("st ");
      return;
    }
    if (lastDigit==2)
    {
      sb.append("nd ");
      return;
    }
    if (lastDigit==3)
    {
      sb.append("rd ");
      return;
    }
    sb.append("th ");
  }

  /**
   * Moves to the next valid address
   */
  public void incrementAddress()
  {
    int addressCount=Address.addressCount();
    if (addressIndex>=addressCount)
      return;
    int lastIndex=addressIndex;    
    do
    {
      addressIndex++;
      if (addressIndex==addressCount)
        if (lastIndex==-1)
        {
          addressEdit.setText("No addresses");
          return;
        }
        else
          addressIndex=0;
    }
    while(!Address.getAddress(addressIndex,address));

    addressLabel.setText("Address "+(addressIndex+1)+" / "+addressCount);
    StringBuffer sb=new StringBuffer();
    if (address.title!=null)
      sb.append(address.title).append(' ');
    if (address.firstName!=null)
      sb.append(address.firstName).append(' ');
    if (address.name!=null)
      sb.append(address.name).append(' ');
    int i;

    for(i=0;i<5&&address.phone[i]==null;i++);
    if (i<5)
      sb.append(address.phone[i]).append(' ');
    else
      sb.append("No Phone! ");

    if (address.note!=null)
      sb.append("(Note)");

    addressEdit.setText(sb.toString());
  }

  /**
   * Moves to the next valid todo
   */
  public void incrementToDo()
  {
    int todoCount=ToDo.todoCount();
    if (todoIndex>=todoCount)
      return;
    int lastIndex=todoIndex;

    do
    {
      todoIndex++;
      if (todoIndex==todoCount)
        if (lastIndex==-1)
        {
          todoEdit.setText("No todos");
          return;
        }
        else
          todoIndex=0;
    }
    while(!ToDo.getToDo(todoIndex,todo));

    todoLabel.setText("ToDo "+(todoIndex+1)+" / "+todoCount);
    StringBuffer sb=new StringBuffer();
    sb.append(todo.completed?"Y ":"N ");
    sb.append(todo.priority).append(' ');
    sb.append(todo.description).append(' ');
    if (todo.note!=null)
      sb.append("(Note)");
    todoEdit.setText(sb.toString());
  }

  /**
   * Moves to the next valid memo
   */
  public void incrementMemo()
  {
    int memoCount=Memo.memoCount();
    if (memoIndex>=memoCount)
      return;
    int lastIndex=memoIndex;

    do
    {
      memoIndex++;
      if (memoIndex==memoCount)
        if (lastIndex==-1)
        {
          memoEdit.setText("No memos");
          return;
        }
        else
          memoIndex=0;
    }
    while(!Memo.getMemo(memoIndex,memo));

    memoLabel.setText("Memo "+(memoIndex+1)+" / "+memoCount);
    memoEdit.setText(memo.text);
  }
}