import waba.ui.*;
import waba.fx.*;
import extra.io.*;

/**
 * A simple test of the BufferStream and DataStream classes.  Simply
 * writes a number of values to a byte array, then extracts them again.
 */
public class StreamTest extends MainWindow
{
  public StreamTest()
  {
    // set out test values
    int intValue=31337;
    int shortValue=-2701;
    float floatValue=3.1415925f;
    String stringValue="Hello";
    boolean booleanValue=true;
    byte byteValue=(byte)200; // note bytes are signed so this goes to -56

    // generate a new sink stream
    BufferStream bs=new BufferStream();
    DataStream ds=new DataStream(bs);

    // write the values
    ds.writeInt(intValue);
    ds.setBigEndian(false);
    ds.writeInt(intValue);
    ds.setBigEndian(true);
    ds.writeShort(shortValue);
    ds.writeFloat(floatValue);
    ds.writeString(stringValue);
    ds.writeCString(stringValue);
    ds.writeFixedString(stringValue,10);
    ds.writeBoolean(booleanValue);
    ds.writeByte(byteValue);
    ds.writeByte(byteValue);

    // get the values written out as a byte array
    byte[] b=bs.getBuffer();

    // use that as a base for another stream
    BufferStream bbs=new BufferStream(b);

    // to make things more interesting, put another stream on top
    // of that one, with a buffer size of 10 bytes
    bs=new BufferStream(bbs,10);
    ds=new DataStream(bs);

    Label l;
    add(l=new Label("readInt="+ds.readInt()+" ("+intValue+")"));
    l.setRect(0,0,160,10);

    ds.setBigEndian(false);
    
    add(l=new Label("readInt (little endian)="+ds.readInt()+" ("+intValue+")"));
    l.setRect(0,12,160,12);

    ds.setBigEndian(true);
    
    add(l=new Label("readShort="+ds.readShort()+" ("+shortValue+")"));
    l.setRect(0,24,160,12);
    add(l=new Label("readFloat="+ds.readFloat()+" ("+floatValue+")"));
    l.setRect(0,36,160,12);
    add(l=new Label("readString=\""+ds.readString()+"\" (\""+stringValue+"\")"));
    l.setRect(0,48,160,12);
    add(l=new Label("readCString=\""+ds.readCString()+"\" (\""+stringValue+"\")"));
    l.setRect(0,60,160,12);
    add(l=new Label("readFixedString=\""+ds.readFixedString(10)+"\" (\""+stringValue+"\")"));
    l.setRect(0,72,160,12);
    add(l=new Label("readBoolean="+ds.readBoolean()+" ("+booleanValue+")"));
    l.setRect(0,84,160,12);
    add(l=new Label("readByte="+ds.readByte()+" ("+byteValue+")")); 
    l.setRect(0,96,160,12);
    add(l=new Label("readUnsignedByte="+ds.readUnsignedByte()+" ("+byteValue+")"));
    l.setRect(0,108,160,12);
  }
}